import tensorflow as tf

def variable_summaries(var, name):
    with tf.name_scope('summaries'):
        with tf.name_scope('mean'):
            mean = tf.reduce_mean(var)
            tf.scalar_summary(name+'/mean', mean)
        with tf.name_scope('stddev'):
            stddev = tf.sqrt(tf.reduce_mean(tf.square(var - mean)))
            tf.scalar_summary(name+'/stdev', stddev)
    tf.scalar_summary(name+'/max', tf.reduce_max(var))
    tf.scalar_summary(name+'/min', tf.reduce_min(var))
    tf.histogram_summary(name, var)

def new_weights(shape):
    return tf.Variable(tf.truncated_normal(shape, stddev=0.05))


def new_biases(length):
    return tf.Variable(tf.constant(0.05, shape=[length]))

def new_conv_layer(input,
                   num_input_channels,
                   filter_size,
                   num_filters,
                   layer_name):
    shape = [filter_size, filter_size, num_input_channels, num_filters]
    with tf.name_scope(layer_name):
        with tf.name_scope('weights'):
            weights = new_weights(shape=shape)
            variable_summaries(weights, layer_name +'/weights')
        with tf.name_scope('biases'):
            biases = new_biases(length=num_filters)
            variable_summaries(biases, layer_name + '/biases')
        layer = tf.nn.conv2d(input=input,
                             filter=weights,
                             strides=[1, 1, 1, 1],
                             padding='SAME')
        layer += biases
        layer = tf.nn.max_pool(value=layer,
                               ksize=[1, 2, 2, 1],
                               strides=[1, 2, 2, 1],
                               padding='SAME')
        layer = tf.nn.relu(layer)
        tf.histogram_summary(layer_name+'/activations', layer)
        return layer


def flatten_layer(layer):
    layer_shape = layer.get_shape()
    num_features = layer_shape[1:4].num_elements()
    layer_flat = tf.reshape(layer, [-1, num_features])

    return layer_flat, num_features


def new_fc_layer(input,
                 num_inputs,
                 num_outputs,
                 layer_name):
    with tf.name_scope(layer_name):
        with tf.name_scope('weights'):
            weights = new_weights(shape=[num_inputs, num_outputs])
            variable_summaries(weights, layer_name + '/weights')
        with tf.name_scope('biases'):
            biases = new_biases(length=num_outputs)
            variable_summaries(weights, layer_name + '/biases')
        layer = tf.matmul(input, weights) + biases
        layer = tf.nn.relu(layer)
        tf.histogram_summary(layer_name+'/activations', layer)
        return layer