import numpy as np
import tensorflow as tf
import mnist
import time
from tensorflow.examples.tutorials.mnist import input_data

# Training board log dirs
SUMMARIES_DIR = '/work/jacoba/tmp/logs'

# Learning parameters
BATCH_SIZE = 100
LEARNING_RATE = 0.01
DROP_PROB = 1.0

def feed(data_set, img_placeholder_flat, lab_placeholder,
         prob_placeholder, prob):
    img_feed, lab_feed = data_set.next_batch(BATCH_SIZE)
    feed_dict = {
        img_placeholder_flat: img_feed,
        lab_placeholder: lab_feed,
        prob_placeholder: prob
    }
    return feed_dict

def do_eval(sess,
            eval,
            img_placeholder,
            lab_placeholder,
            prob_placeholder,
            data_set):
    true_count = 0
    steps_per_epoch = data_set.num_examples // BATCH_SIZE
    num_examples = data_set.num_examples

    # Loop predicts labels from all given examples
    for step in range(steps_per_epoch):

        # Dict to give data to the graph
        feed_dict = feed(data_set, img_placeholder,
                         lab_placeholder=lab_placeholder,
                         prob_placeholder= prob_placeholder,
                         prob = 1.0)
        true_count += sess.run(eval, feed_dict = feed_dict)

    accuracy= true_count/num_examples
    print(' Num examples: %d, Num correct: %d, Test error rate: %0.04f' %
              (num_examples , true_count, 100*(1-accuracy)))

def run_training(steps):
    data_set = input_data.read_data_sets('data/')


    img_placeholder, img_placeholder_flat, lab_placeholder,\
        prob_placeholder = mnist.placeholder_inputs(batch_size = BATCH_SIZE)
    logits = mnist.inference(img_placeholder, prob_placeholder)
    loss = mnist.loss(logits, lab_placeholder)
    train = mnist.training(loss, learning_rate = LEARNING_RATE)
    eval = mnist.evaluation(logits, lab_placeholder)

    # Add scalar summary for loss
    tf.scalar_summary("loss", loss)
    summary = tf.merge_all_summaries()
    init = tf.initialize_all_variables()
    sess = tf.Session()
    summary_writer = tf.train.SummaryWriter(SUMMARIES_DIR, graph=sess.graph)
    sess.run(init)

    # Training loop
    for step in range(steps):
        start_time = time.time()
        feed_dict = feed(data_set.train, img_placeholder_flat=img_placeholder_flat,
                         lab_placeholder=lab_placeholder,
                         prob_placeholder=prob_placeholder,
                         prob=DROP_PROB)

        # Get loss value from graph
        _, summary_str = sess.run([train, summary], feed_dict = feed_dict)
        summary_writer.add_summary(summary_str, step)

        if step % 100 == 0:
            print('Step: %d' % (step))

        if step % 1000 == 0:
            print('Training data evaluation:')
            do_eval(sess,
                    eval,
                    img_placeholder=img_placeholder_flat,
                    lab_placeholder=lab_placeholder,
                    prob_placeholder=prob_placeholder,
                    data_set=data_set.train)

            print('Test data evaluation:')
            do_eval(sess,
                    eval,
                    img_placeholder=img_placeholder_flat,
                    lab_placeholder=lab_placeholder,
                    prob_placeholder=prob_placeholder,
                    data_set=data_set.test)

run_training(1001)






